#! /bin/bash
#

# Messages functions

if tty -s < /dev/stdout ; then TTY_OUT=1; else TTY_OUT=0 ; fi

msg-white()
{
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[37;2m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}
msg-red(){
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[31;2m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}
msg-green(){
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[32;2m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}
msg-blue(){
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[34;2m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}
msg-yellow(){
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[33;2m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}

msg-white-rv(){
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[37;2;7m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}
msg-red-rv(){
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[31;2;7m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}
msg-green-rv(){
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[32;2;7m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}
msg-blue-rv(){
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[34;2;7m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}
msg-yellow-rv(){
    if [[ $TTY_OUT -eq 1 ]]; then printf '\E[33;2;7m%s\E[0m\n' "$*"
    else printf '%s\n' "$*" ; fi
}


export PATH=/bin:/sbin:/usr/bin:/usr/sbin:$PATH

if [ "$#" -ge 2 -a "$1" = "-m" -a -s "$2" ]
then
	dirpart=$(dirname $BASH_SOURCE)
	if [ "$dirpart" = "." ]; then dirpart=$PWD; fi
	cmd=$(basename $BASH_SOURCE)
	typeset -i nhost=0
	hfile="$2"
	echo Running "$cmd" across all hosts in file "$hfile" using ssh.
	hosts=$(cat "$2")
	echo Host list is $hosts
	for h in ${hosts}
	do
		echo Checking host $h
		ssh -x -o StrictHostKeyChecking=no $h /bin/bash -f << EOF
		h=$(hostname);
		PATH=$PATH:$dirpart
		$cmd | sed "s/^/${h}: /"
EOF
		nhost=$(($nhost + 1))
	done
	echo Completed running "$cmd" across $nhost hosts from "$hfile"
	exit 0
fi
msg-blue "Date:" 
date
msg-blue Host name: 
hostname
msg-blue System Kernel:
uname -r


msg-yellow-rv "===== HCA card parameters: ====="


echo ""
msg-blue "Check to see if HCA hardware is present:"
/sbin/lspci | grep InfiniHost | sed "s/^/    /"

msg-green "============================================================"
echo ""
msg-blue "Read HCA information from the driver:"
/usr/bin/vstat


msg-green "============================================================"

echo ""
msg-blue "Capture HCA driver and chip counter values:"
/usr/mellanox/bin/get_pcounter


msg-green "============================================================"
echo ""
echo ""
msg-yellow-rv "===== Voltaire Host stack debug information ====="
echo ""
msg-blue "Checking for Voltaire RPMs:"
/bin/rpm -qa | sort| egrep -i "ibhost" | sed "s/^/    /"
echo ""
msg-blue "Check if IB-Host modules are loaded in the kernel:"
lsmod | sort | egrep -i "itarget |kdapltest_module |kdapl | kdat_registry |ipoib_ud |ats |devugsi |devucm |ibat |cm |gsim |ad_tavor |vverbs |mlog |repository |hadump |mod_ib_mgt |mod_thh |mod_vip |mlxsys " 

module_count=`lsmod | sort | egrep -i "itarget |kdapltest_module |kdapl | kdat_registry |ipoib_ud |ats |devugsi |devucm |ibat |cm |gsim |ad_tavor |vverbs |mlog |repository |hadump |mod_ib_mgt |mod_thh |mod_vip |mlxsys " |wc -l`
echo ""
msg-red "$module_count modules were found"
msg-green "============================================================"
if (( $module_count < 18 )) ; then
	msg-red "Missing modules. please verify that stack is properly installed"
fi

echo ""
msg-blue "Check if dmesg contain any errors"

dmesg |grep -i error| sed "s/^/    /"

msg-green "============================================================"
echo ""
msg-blue "Check the server networking:"
msg-blue "	Check first for file entries:"
egrep -is "vapi" /etc/modprobe.conf /etc/modprobe.conf.vapi | sed "s/^/    /"
msg-blue "	Check ifconfig output:"
/sbin/ifconfig | sed "s/^/    /"


