#!/bin/bash
# Created 19/11/2003
# Updated may 2005
# Updated 10/28/2005 by Erez Strauss <erezs@voltaire.com>

. "${basedir}"/ib-check-cluster-mpi.sh

function nodes-read-list()
{
    # H - Hosts array.
    H=( $(cat $nodes_file | sort -u ) )
}
function nodes-show()
{
    for (( i=0; i< ${#H[@]}; i++)); do
        echo "host[$i]: ${H[$i]}"
    done
}
function nodes-show-seq-pairs()
{
    for (( i=0; i<${#H[@]}; i++)); do
        echo "pair[$i] ${H[$i]} ${H[$(( (i + 1) % ${#H[@]} ))]}"
    done
}
function nodes-show-all-pairs()
{
    # does not scale for testing real clusters, unless running N jobs in background.
    local i j
    for (( i=0; i<${#H[@]}; i++)); do
        for (( j=0; j<${#H[@]}; j++)); do
            if [[ $i -ne $j ]]; then
                echo "pair[$i,$j] ${H[$i]} ${H[$j]}"
            fi
        done
    done
}
function ib-check-cluster-seq-pairs-paralel-ping()
{
# Update to use the ipoib0 IP addresses.
    local rc=0
    for (( i=0; i<${#H[@]}; i++)); do
        h1=${H[$i]}
        h2=${H[$(( (i + 1) % ${#H[@]} ))]}
	$rsh_cmd -n ${h1} "ping ${IBIP[$(( (i + 1) % ${#H[@]} ))]} -c 1000 -f -w 15 | grep loss" > "${tempdir}"/ib-check-$$-$h1-ping.log 2>&1 &
    done
    wait
    for (( i=0; i<${#H[@]}; i++)); do
        h1=${H[$i]}
        h2=${H[$(( (i + 1) % ${#H[@]} ))]}
	lost=$(awk '/loss/{print $6}' "${tempdir}"/ib-check-$$-$h1-ping.log)
	if [[ "$lost" != "0%" ]]; then
	    msg-red "Error: ping $h1 -> $h2 lost $lost precent of the packets"
	    rc=1
	else
	    msg-green "ping $h1 -> $h2 works well"
        fi
    done
    return $rc
}
function ib-check-cluster-perf-main-bw-loopback()
{
    for (( i=0; i< ${#H[@]}; i++)); do
        local host=${H[$i]}
	$rsh_cmd -n $host "killall perf_main"
    done
    sleep 1
    for (( i=0; i< ${#H[@]}; i++)); do
        local host=${H[$i]}
	$rsh_cmd -n $host ". /etc/profile ; perf_main --send -mbw -rwrite -s256000 -n500" > "${tempdir}/ib-check-$$-${host}-perf-main-bw-lb.log" 2>&1 &
    done
    sleep 1
    for (( i=0; i< ${#H[@]}; i++)); do
        local host=${H[$i]}
        $rsh_cmd -n $host ". /etc/profile ; perf_main -a 127.0.0.1" > "${tempdir}/ib-check-$$-${host}-perf-main-bw-lb-rcv.log" 2>&1 &
    done
    wait
    sync
    # Sort results and search for problematic node(s)
    msg-white "Loopback bandwidth:"
    grep 256000 "${tempdir}"/ib-check-$$-*-perf-main-bw-lb.log | sort +1n
}
function ib-check-cluster-perf-main-bw-parallel()
{
    for (( i=0; i< ${#H[@]}; i++)); do
        local host=${H[$i]}
	$rsh_cmd -n $host "killall perf_main"
    done
    sleep 1
    for (( i=0; i + 1 < ${#H[@]}; i += 2)); do
        h1=${H[$i]}
        h2=${H[$(( (i + 1) % ${#H[@]} ))]}
	$rsh_cmd -n $h1 ". /etc/profile ; perf_main --send -mbw -rwrite -s256000 -n500" > "${tempdir}/ib-check-$$-${h1}-${h2}-perf-main-bw-snd.log" 2>&1 &
    done
    sleep 1
    for (( i=0; i + 1 < ${#H[@]}; i += 2)); do
        h1=${H[$i]}
        h2=${H[$(( (i + 1) % ${#H[@]} ))]}
        $rsh_cmd -n $h2 ". /etc/profile ; perf_main -a ${EIP[$i]}" > "${tempdir}/ib-check-$$-${h1}-${h2}-perf-main-bw-rcv.log" 2>&1 &
    done
    wait
    msg-white "perf_main bandwidth (in paralell):"
    grep 256000 "${tempdir}"/ib-check-$$-*-perf-main-bw-snd.log | sort +1n
}
function ib-check-cluster()
{
    # On Each node run the following:
    # 1. hostname - verify the rsh/ssh mechanism & hostname resolving of the head
    # 2. Check for ibhost-release file content (rpm -V ?)
    #x3. rpm verification
    #x4. lspci
    #x5. ibhost.init reset
    #x6. vstat
    # 7. IPoIB address & ping
    # 8. perf_main
    # 9. mpi tests if mpi_tests != 0
    #
    # At the end report if not all nodes are active

    msg-white "Voltaire InfiniBand Stack - Cluster Verification"
    nodes-read-list
    msg-white "There are ${#H[@]} nodes in your node list (${nodes_file})"

    # 1. Check for ssh/rsh functionality.
    msg-white "Testing rsh/ssh using hostname"
    err_count=0
    for (( i = 0; i < ${#H[@]}; i++ )); do
        host=${H[$i]}
        h_name="$($rsh_cmd -n $host hostname)"
        if [[ $? -ne 0 ]]; then
            msg-red "failed to get host name from '$host'."
            (( err_count++ ))
        fi
        if [[ $host != "$h_name" ]]; then
            msg-red "Possible host name conflicts '$host' vs. '$h_name'"
        fi
        local h_e_ipaddr="$($rsh_cmd -n $host ifconfig eth0 | awk '/inet addr:/{print $2}' | sed s/addr://)"
	EIP[$i]="${h_e_ipaddr}"
        local h_ib_ipaddr="$($rsh_cmd -n $host ifconfig eth0 | awk '/inet addr:/{print $2}' | sed s/addr://)"
	IBIP[$i]="${h_ib_ipaddr}"
    done
    if [[ err_count -eq 0 ]]; then
        msg-green "All nodes reported their hostname"
    else
        msg-red "Error: Rsh/ssh probleam on $err_count nodes"
    fi

    # Check RPM package
    if [[ $rpm_check -eq 1 ]]; then
	for (( i = 0; i < ${#H[@]}; i++ )); do
	    host=${H[$i]}
	    $rsh_cmd -n $host 'rpm -qva | grep ibhost-' > "${tempdir}/ib-check-$$-${host}-rpm.log" 2>&1 &
	done
	wait
	for (( i = 0; i < ${#H[@]}; i++ )); do
	    host=${H[$i]}
	    rpm_package="$(cat ${tempdir}/ib-check-$$-${host}-rpm.log)"
	    if [[ $? -ne 0 ]]; then
		msg-red "[$host] Failed to get RPM packages"
	        (( err_count++ ))
	    fi
	    if [[ $(echo "$rpm_package" | wc -l) -eq 1 ]]; then
		msg-green "[$host] Found $rpm_package installed"
		rpm_v="$($rsh_cmd -n $host rpm -V $rpm_package 2>&1 | grep -v repository | grep -v mst.conf)"
		if [[ $(echo "$rpm_v" | wc -c) -gt 10 ]]; then
		    msg-red "[$host] Warning: The following files of the $rpm_package were modifyed: $rpm_v"
		fi
	    else 
		msg-red "[$host] Error: Number of ibhost packages is *not* one: $rpm_package."
		return 1
	    fi
	done
	if [[ err_count -eq 0 ]]; then
	    msg-green "All nodes have installed IB packaged"
	else
	    msg-red "Error: not All node have valid IB stack, $err_count errornous nodes"
	    (( rc++ ))
	fi
    fi

    # Check lspci on all nodes
    err_count=0
    for (( i = 0; i < ${#H[@]}; i++ )); do
        host=${H[$i]}
        lspci_mlx="$($rsh_cmd -n $host '/sbin/lspci | grep Mellanox | wc -l')"
        if [[ $? != 0 ]]; then
            (( err_count++ ))
        fi
        if [[ $lspci_mlx -ne 1 && $lspci_mlx -ne 2 ]]; then
            msg-red "[$host] Error: lspci does *not* show HCA card"
        else
            msg-green "[$host] lspci found HCA card"
        fi
    done

    # IB host restart
    #
    for (( i = 0; i < ${#H[@]}; i++ )); do
	host=${H[$i]}
	$rsh_cmd -n $host ". /etc/profile ; if [[ ${use_script} -ne 0 && -e /usr/bin/script ]]; then  script -c 'ibhost.init restart' /dev/stdout ; else ibhost.init restart ; fi" > ${tempdir}/ib-check-$$-${host}-ibrestart.log 2>&1 &
    done
    wait
    for (( i = 0; i < ${#H[@]}; i++ )); do
        host=${H[$i]}
        ib_restart="$(cat ${tempdir}/ib-check-$$-${host}-ibrestart.log)"
        if [[ $? != 0 ]]; then
	    msg-red "[$host] Failed to restart IB stack"
            (( err_count++ ))
	fi
	sleep 1
	host_dmesg="$($rsh_cmd -n $host dmesg)"
	if ! echo "$host_dmesg" | grep -i thh | grep -iq failed ; then
            msg-green "[$host] dmesg shows no errors on stack restart (load)"
        else
            msg-red "[$host] Error: dmesg shows errors on stack restart, please use dmesg to get details"
            msg-red "[$host] Error: dmesg shows there are errors restarting the IB host stack.  Please review.  When the issue is resolved you can run ib-check.sh again."
        fi
    done
     
    # IB Stack status verification
    #
    err_count=0
    for (( i = 0; i < ${#H[@]}; i++ )); do
        host=${H[$i]}
        h_vstat="$($rsh_cmd -n $host '. /etc/profile ; if [[ -e /usr/mellanox/bin/vstat ]]; then /usr/mellanox/bin/vstat ; else /usr/bin/vstat ; fi ')"
        if [[ $? != 0 ]]; then
            (( err_count++ ))
        fi
        #
        ib-vstat-parse-info "$host" "$h_vstat"
    done

    # 7. IPoIB address & ping
    ib-check-cluster-seq-pairs-paralel-ping
    
    # Loopback performance using perf_main.
    ib-check-cluster-perf-main-bw-loopback

    # 8. perf_main
    ib-check-cluster-perf-main-bw-parallel

    # 9. mpi tests if mpi_tests != 0
    if [[ $vmpi_tests -ne 0 ]]; then
	ib-check-cluster-mpi-tests
    fi

    return 0
}

# Script Testing Function

function ib-check-test-script ()
{
    nodes-show-seq-pairs
    nodes-show-all-pairs
    test_seq_pairs_bi_directional
    test_seq_pairs_uni_directional
}

# main

ib-check-cluster

